{% extends "basic_template.html" %}
{% block title %}
{{ super() }}
{% endblock %}
{% block head %}
{{ super() }}
{% endblock %}
{% block content %}

<h1>Results</h1>


<div class="w3-row-padding">

    <div class="w3-container w3-margin w3-border-large">
        <div style="overflow-x: auto">
            <table class="w3-table-all w3-centered w3-hoverable">
                <caption class="w3-container w3-us-blue"><h3>Fixed points/Steady states</h3></caption>
                <tr class="w3-us-blue">
                    {% for variable in variables %}
                    <th>{{ variable }}</th>
                    {% endfor %}
                </tr>
                {% for fixed_point in fixed_points %}
                <tr>
                    {% for val in fixed_point %}
                    <td>{{ val }}</td>
                    {% endfor %}
                </tr>
                {% endfor %}
            </table>
        </div>
    </div>

    <form action="/download-tsv/" method="POST">
    <textarea name="model_result" style="display: none;">{% for variable in variables %}{{ variable }}	{% endfor %}
{% for fixed_point in fixed_points %}{% for val in fixed_point %}{{ val }}	{% endfor %}
{% endfor %}
</textarea>
        <input class="w3-btn w3-us-red w3-margin w3-round" type="submit" value="Download tsv">
    </form>


</div>

{% endblock %}
