{% extends "basic_template.html" %}
{% block title %}Index{% endblock %}
{% block head %}
  {{ super() }}
{% endblock %}
{% block content %}

<h1>About the Authors</h1>

<section>
  <div class="w3-row-padding">
    
    <div class="w3-quarter">
      <div class="w3-card-4 w3-dark-grey w3-center w3-margin w3-padding-large" style="width:80%">
	<div class="w3-container w3-center">
	  <img src="{{ url_for('static', filename='AdamKnapp.jpg') }}" class="w3-circle" alt="Adam Knapp" style="width:80%; max-width:200px; max-height:200px">
	  <h5>Adam Knapp</h5>
	  <div class="w3-section">
	      Developer<br>
		  Division of Pulmonary, Critical Care, and Sleep Medicine<br>
		  Department of Medicine<br>
		  University of Florida<br>
	      <a href="mailto:adam.knapp@medicine.ufl.edu">Email</a>
	  </div>
	</div>      
      </div>
    </div>
    
    <div class="w3-quarter">
      <div class="w3-card-4 w3-dark-grey w3-center w3-margin w3-padding-large" style="width:80%">
	<div class="w3-container w3-center">
	  <img src="{{ url_for('static', filename='JuliaChifman.jpg') }}" class="w3-circle" alt="Julia Chifman" style="width:80%; max-width:200px; max-height:200px">
	  <h5>Julia Chifman</h5>
	  <div class="w3-section">
	    Department of Mathematics and Statistics<br>
		  American University<br>
	    <a href="mailto:chifman@american.edu">Email</a>
	  </div>
	</div>
      </div>
    </div>

    <div class="w3-quarter">
      <div class="w3-card-4 w3-dark-grey w3-center w3-margin w3-padding-large" style="width:80%">
	<div class="w3-container w3-center">
	  <img src="{{ url_for('static', filename='LuisSV.png') }}" class="w3-circle" alt="Luis Sordo Vieira" style="width:80%; max-width:200px; max-height:200px">
	  <h5>Luis Sordo Vieira</h5>
	  <div class="w3-section">
		  Division of Pulmonary, Critical Care, and Sleep Medicine<br>
		  Department of Medicine<br>
		  University of Florida
	  </div>
	</div>
      </div>
    </div>

    <div class="w3-quarter">
      <div class="w3-card-4 w3-dark-grey w3-center w3-margin w3-padding-large" style="width:80%">
	<div class="w3-container w3-center">
	  <img src="{{ url_for('static', filename='ReinhardLaubenbacher.jpg') }}" class="w3-circle" alt="Reinhard Laubenbacher" style="max-width:200px; max-height:200px">
	  <h5>Reinhard Laubenbacher</h5>
	  <div class="w3-section">
		Dean’s Professor of Systems Medicine<br>
		Director, Laboratory for Systems Medicine<br>
		Division of Pulmonary, Critical Care, and Sleep Medicine<br>
		Department of Medicine<br>
		University of Florida
	  </div>
	</div>
      </div>
    </div>

    
  </div>
</section>
            
{% endblock %}
