{% extends "basic_template.html" %}
{% block title %}
{{ super() }}
{% endblock %}
{% block head %}
{{ super() }}
{% endblock %}
{% block content %}

<h1>Documentation</h1>

<p>This is where the documentation would go.</p>

<p>We should describe
<ol>
    <li> the web interface</li>
    <li> the command line tools</li>
    <li> something else?</li>
</ol>
</p>


{% endblock %}
